%CITATION Strohsal, T., Proao, C.R., Wolters, J. (2015): 
%"Characterizing the Financial Cycle: Evidence from a Frequency Domain Analysis"
%Discussion Paper, Deutsche Bundesbank, No 22/2015.

%% get data
%read estimated coefficients
est_parameters=zeros(13,2,2);
boot_parameters=zeros(13,10000,2);

fid=fopen('ESTCOEFS_HOUSING_PRE.txt', 'r');
a=textscan(fid, '%f %f %f %f %f %f %f %f %f %f %f %f %f');
est_parameters(:,:,1)=cell2mat(a)';
fclose(fid);
fid=fopen('ESTCOEFS_HOUSING_POST.txt', 'r');
a=textscan(fid, '%f %f %f %f %f %f %f %f %f %f %f %f %f');
est_parameters(:,:,2)=cell2mat(a)';
fclose(fid);

fid=fopen('BOOTCOEFS_HOUSING_PRE.txt', 'r');
a=textscan(fid, '%f %f %f %f %f %f %f %f %f %f %f %f %f');
boot_parameters(:,:,1)=cell2mat(a)';
fclose(fid);
fid=fopen('BOOTCOEFS_HOUSING_POST.txt', 'r');
a=textscan(fid, '%f %f %f %f %f %f %f %f %f %f %f %f %f');
boot_parameters(:,:,2)=cell2mat(a)';
fclose(fid);


%% produce statistics (point estimates) from main results table
frequency_fig=zeros(250,2);
spectral_density_fig=zeros(250,2);
frequency_max_years=zeros(1,2);
mass_at_peak=zeros(1,2);
frac_2_to_8=zeros(1,2);
frac_8_to_32=zeros(1,2);

for i=1:2
    y=est_parameters(:,:,i);
    %define parameters
    ar=[1 -1*y(1:12,1)'];
    ma=[1 y(1:12,2)'];
    grid=999;
    residvar=y(13,1)^2;
    xvar=y(13,2)^2;
    %get spectra and spectral densities
    [frequency,spectrum]=time_to_frequency(ar,ma,grid,residvar);
    spectral_density=spectrum./((pi/1000)*(trapz(spectrum)*2));
    %get main cycle
    idx=find(spectrum==max(spectrum));
    frequency_max=frequency(idx);
    frequency_max_years(1,i)=pi2years(frequency_max);
    %get spectral mass around main cycle
    mass_at_peak(1,i)=get_mass_at_peak(spectrum,frequency,frequency_max);
    %get spectral mass between 2 and 8 years
    total_mass=trapz(spectrum);
    idx8 = arrayfun(@(x) find(frequency>=x,1), years2pi(8));
    idx2 = arrayfun(@(x) find(frequency>=x,1), years2pi(2));
    idx32 = arrayfun(@(x) find(frequency>=x,1), years2pi(32));
    mass_2_to_8=trapz(spectrum(idx8:idx2));
    frac_2_to_8(1,i)=(mass_2_to_8/total_mass)*100;
    %get mass of variance between 8 and 32 years
    mass_8_to_32=trapz(spectrum(idx32:(idx8+1)));
    frac_8_to_32(1,i)=(mass_8_to_32/total_mass)*100;
    %cut off higher frequencies for figures
    frequency_fig(:,i)=frequency(1:250);
    spectral_density_fig(:,i)=spectral_density(1:250);
end


%% create figure
filenames_country={'GER','GER'};
filenames_timeseries={'Housing','Housing'};

i=1;
fig=figure;
pl1=plot(frequency_fig(:,i),spectral_density_fig(:,i),'Color','k','linestyle','-.','LineWidth',2); %#ok<NASGU>
%set(gca,'TickLabelInterpreter', 'tex');
set(gca,'XTick',[0 0.049 0.196 0.393 0.589 0.782])
set(gca,'XTickLabel', {'' '\pi/64' '\pi/16' '\pi/8' '3\pi/16' '\pi/4'},'fontsize',20);
axis tight
ylim([0 1.01*max([spectral_density_fig(:,i); spectral_density_fig(:,(i+1))])])
hold on
y_max_area=1.002*max([spectral_density_fig(:,i); spectral_density_fig(:,(i+1))]);
x_max_area=0.0075*max([spectral_density_fig(:,i); spectral_density_fig(:,(i+1))]);
x = [0.051 0.051 0.1945 0.1945]; y = [x_max_area y_max_area y_max_area x_max_area]; % define edges of shaded area
area_handle=fill([x(1) x(2) x(3) x(4)], [y(1) y(2) y(3) y(4)], [0.9 0.9 0.9]); % put shaded area
set(area_handle,'edgecolor','none');
pl1=plot(frequency_fig(:,i),spectral_density_fig(:,i),'Color','k','linestyle','-.','LineWidth',2);
axis tight
ylim([0 1.01*max([spectral_density_fig(:,i); spectral_density_fig(:,(i+1))])])
xlabel('Frequency','fontname','Arial','fontsize',20)
ylabel('Spectral Density','fontname','Arial','fontsize',20)
pl2=plot(frequency_fig(:,(i+1)),spectral_density_fig(:,(i+1)),'Color','k','LineWidth',2);
legend([pl1,pl2],{'Before 1990Q2','After 1990Q2'},'Location','NorthEast','fontname','Arial','fontname','Arial','fontsize',20)
title(strcat(filenames_country(i),'{ }',filenames_timeseries(i)),'fontname','Arial','fontsize',20,'FontWeight','bold')
graphname=strcat(filenames_country(i),' ',filenames_timeseries(i));
print(graphname{1},'-dpdf')


%% get 95% confidence intervals
boot_frequency_max_years=zeros(1,5000,2);
boot_mass_at_peak=zeros(1,5000,2);
boot_frac_2_to_8=zeros(1,5000,2);
boot_frac_8_to_32=zeros(1,5000,2);
aux_idx=reshape([1:5000; 1:5000],[1, 10000]);

for i=1:2:9999
    for s=1:2
        y=boot_parameters(:,i:i+1,s);
        ar=[1 -1*y(1:12,1)'];
        ma=[1 y(1:12,2)'];
        residvar=y(13,1)^2;
        xvar=y(13,2)^2;
        [frequency,spectrum]=time_to_frequency(ar,ma,grid,residvar);
        spectral_density=spectrum./((pi/1000)*(trapz(spectrum)*2));
        idx=find(spectrum==max(spectrum));
        frequency_max=frequency(idx);
        boot_frequency_max_years(1,aux_idx(i),s)=pi2years(frequency_max);
        boot_mass_at_peak(1,aux_idx(i),s)=get_mass_at_peak(spectrum,frequency,frequency_max);
        total_mass=trapz(spectrum);
        idx8 = arrayfun(@(x) find(frequency>=x,1), years2pi(8));
        idx2 = arrayfun(@(x) find(frequency>=x,1), years2pi(2));
        idx32 = arrayfun(@(x) find(frequency>=x,1), years2pi(32));
        mass_2_to_8=trapz(spectrum(idx8:idx2));
        boot_frac_2_to_8(1,aux_idx(i),s)=(mass_2_to_8/total_mass)*100;
        mass_8_to_32=trapz(spectrum(idx32:(idx8+1)));
        boot_frac_8_to_32(1,aux_idx(i),s)=(mass_8_to_32/total_mass)*100;
    end
end

%delete (near) unit root versions here
idx_stationary=zeros(1,5000,2);
for i=1:2:9999
    for s=1:2
        idx_stationary(1,aux_idx(i),s)=sum(boot_parameters(1:12,i,s))<0.995;
    end
end
boot_frequency_max_years_pre=boot_frequency_max_years(:,idx_stationary(1,:,1)==1,1);
boot_frequency_max_years_post=boot_frequency_max_years(:,idx_stationary(1,:,2)==1,2);
boot_mass_at_peak_pre=boot_mass_at_peak(:,idx_stationary(1,:,1)==1,1);
boot_mass_at_peak_post=boot_mass_at_peak(:,idx_stationary(1,:,2)==1,2);
boot_frac_2_to_8_pre=boot_frac_2_to_8(:,idx_stationary(1,:,1)==1,1);
boot_frac_2_to_8_post=boot_frac_2_to_8(:,idx_stationary(1,:,2)==1,2);
boot_frac_8_to_32_pre=boot_frac_8_to_32(:,idx_stationary(1,:,1)==1,1);
boot_frac_8_to_32_post=boot_frac_8_to_32(:,idx_stationary(1,:,2)==1,2);

%calculate quantiles
boot_frequency_max_years_pre_interval=quantile(boot_frequency_max_years_pre, [0.025 0.975]);
boot_frequency_max_years_post_interval=quantile(boot_frequency_max_years_post, [0.025 0.975]);
boot_mass_at_peak_pre_interval=quantile(boot_mass_at_peak_pre, [0.025 0.975]);
boot_mass_at_peak_post_interval=quantile(boot_mass_at_peak_post, [0.025 0.975]);
boot_frac_2_to_8_pre_interval=quantile(boot_frac_2_to_8_pre, [0.025 0.975]);
boot_frac_2_to_8_post_interval=quantile(boot_frac_2_to_8_post, [0.025 0.975]);
boot_frac_8_to_32_pre_interval=quantile(boot_frac_8_to_32_pre, [0.025 0.975]);
boot_frac_8_to_32_post_interval=quantile(boot_frac_8_to_32_post, [0.025 0.975]);


%% print results
formatSpec = ['Main cycle length pre is %4.2f with 95%% interval [%4.2f %4.2f],\n'...
    'main cycle length post is %4.2f with 95%% interval [%4.2f %4.2f],\n'...
    'spectral mass at main cycle pre is %5.2f percent with 95%% interval [%5.2f %5.2f],\n'...
    'spectral mass at main cycle post is %5.2f percent with 95%% interval [%5.2f %5.2f],\n'...
    'spectral mass 8 to 32 years pre is %5.2f percent with 95%% interval [%5.2f %5.2f],\n'...
    'spectral mass 8 to 32 years post is %5.2f percent with 95%% interval [%5.2f %5.2f],\n'...
    'spectral mass 2 to 8 years pre is %5.2f percent with 95%% interval [%5.2f %5.2f],\n'...
    'spectral mass 2 to 8 years post is %5.2f percent with 95%% interval [%5.2f %5.2f].'] ;
fprintf(formatSpec,frequency_max_years(1), boot_frequency_max_years_pre_interval,...
    frequency_max_years(2), boot_frequency_max_years_post_interval,...
    mass_at_peak(1), boot_mass_at_peak_pre_interval,...
    mass_at_peak(2), boot_mass_at_peak_post_interval,...
    frac_8_to_32(1), boot_frac_8_to_32_pre_interval,...
    frac_8_to_32(2), boot_frac_8_to_32_post_interval,...
    frac_2_to_8(1), boot_frac_2_to_8_pre_interval,...
    frac_2_to_8(2), boot_frac_2_to_8_post_interval);